const { h } = require('preact');
const { join, posix } = require('path');
const { EOL } = require('os');
const { ju } = require('../lib');
const { join: pjoin } = posix

const DEFAULT_TO = '.documentary/section-breaks'

/**
 * Writes method title. Not exported, has to be invoked manually.
 * @param {Object} opts
 * @param {number} opts.number The counting number of the section break. Resets after 22.
 * @param {string} opts.url The URL of the directory where to find the section breaks.
 * @param {string} opts.to The location where to save the section break.
 * @param {string} opts.href Where should the user be taken when clicking on the break.
 * @param {import('../lib/Documentary').default} opts.documentary
 */
function SectionBreak({ number, url = '', documentary, alt = 'section break', to = DEFAULT_TO,
  href = '#table-of-contents', children, ...attrs }) {
  let n = 0
  const { sectionBrakeNumber, SECTION_BREAKS_URL = url, _args: { wiki, gitlab }, pretty, PUBLIC_URL,
    copyFileSync, PUBLIC_DIR,
  } = documentary
  pretty(false)
  if (number) n = parseInt(number)
  else if (sectionBrakeNumber !== undefined) n = (sectionBrakeNumber + 1) % 23

  const name = `${n}.svg`
  const imgPath = join(__dirname, '../section-breaks', name)

  let src

  if (SECTION_BREAKS_URL.startsWith('public://')) {
    const p = SECTION_BREAKS_URL.replace('public://', '')
    if (!PUBLIC_URL) throw new Error(`SECTION_BREAKS_URL is set to ${
      SECTION_BREAKS_URL
    }, but PUBLIC_URL wasn't set on Documentary. Use <public> component to set the PUBLIC path.`)
    const publicFile = join(PUBLIC_DIR, p, name)
    copyFileSync(imgPath, publicFile)
    src = ju(SECTION_BREAKS_URL, name)
  } else if (!SECTION_BREAKS_URL) {
    const toFs = wiki ? join(wiki, to) : to
    const FILE = join(toFs, name)
    copyFileSync(imgPath, FILE)

    const nn = `${name}${(gitlab || wiki) ? '' : '?sanitize=true'}`
    src = pjoin('/', to, nn)
  } else src = ju(SECTION_BREAKS_URL, name)

  const positive = n >= 0
  if (positive) documentary.sectionBrakeNumber = n

  attrs['alt'] = alt

  let img
  if (wiki) {
    const a = Object.entries(attrs).reduce((acc, [k, val]) => {
      if (!val) return acc
      if (val === true) return `${acc}|${k}`
      return `${acc}|${k}=${val}`
    }, '')
    img = `[[${src}${a}]]`
  } else img = (h('img',{...attrs,'src':src }))
  return (         h(Wrapper,{href:href},
    img,
  ))
}


const Wrapper = ({ href, children }) => {
  let [img] = children
  let a
  if (typeof img == 'string') {
    img = `${EOL}  ${img}${EOL}`
    a = (h('a',{'href':href, 'dangerouslySetInnerHTML':{ __html: img } }))
  } else {
    a =  h('a',{'href':href},
      `${EOL}  `,img,EOL,
    )
  }
  return (h('div',{'align':"center"},h('p',{'align':"center"},a)))
}

module.exports = SectionBreak