const { c, b } = require('../../stdlib');
const { competent } = require('../../stdlib');
const Components = require('../components/');
const Method = require('../components/method');

/**
 * @param {string[]} paths Paths where to look for components.
 * @param {*} getDocumentary
 */
const getComponents = (paths, getDocumentary) => {
  let method = Method, SECTION_BREAKS_URL
  const transforms = paths.reduce((acc, path) => {
    try {
      const required = require(path) // an object
      Object.entries(required).forEach(([key, e]) => {
        if (key == 'method') {
          method = e
          return
        } else if (key == 'SECTION_BREAKS_URL') {
          SECTION_BREAKS_URL = e
          return
        }
        if (acc[key] && acc[key] !== e) console.error('Overriding component <%s> by new one from %s', key, path)
        acc[key] = e
      })
      return acc
    } catch (err) {
      if (!err.message.includes(`Cannot find module '${path}'`)) {
        console.log(err.stack)
      }
      return acc
    }
  }, Components)
  const components = createRules(transforms, getDocumentary)
  return { method, components, SECTION_BREAKS_URL }
}

/**
 * Uses competent to setup rules for restream.
 * @param {*} components
 * @param {() => Documentary} getDocumentary Returns the documentary instance.
 */
function createRules(components, getDocumentary) {
  const rule = competent(components, {
    onFail(key, err) {
      console.error('%s %s', c('Could not process component', 'yellow'), c(b(`<${key}>`, 'yellow'), 'white'))
      const reg = new RegExp(`^ +at ${key} .+`, 'im')
      let i
      const stack = err.stack.replace(reg, (mm, j) => {
        i = j + mm.length
        return mm
      }).slice(0, i)
      const d = getDocumentary()
      d.error({ stack, message: err.message })
    },
    /**
     * @param {Object} htmlProps
     * @param {import('competent').Meta} meta
     */
    getProps(htmlProps, meta, name) {
      meta.setPretty(true, 100)
      const d = getDocumentary()
      const { currentFile } = d
      const documentary = new Proxy(d, {
        get(target, p) {
          switch (p) {
          case 'currentFile': return currentFile
          case 'renderAgain': return meta.renderAgain
          case 'setPretty': return meta.setPretty
          case 'pretty': return meta.setPretty
          case 'removeLine': return () => {
            meta.removeLine()
            return null
          }
          case 'error': return (err) => {
            const stack = err.stack.replace(err.message, '')
            console.error(b(`<${name}>`, 'yellow'), c(err.message, 'red'))
            console.error(c(stack, 'grey'))
          }
          }
          let pp = target[p]
          if (typeof pp == 'function') pp = pp.bind(documentary)
          return pp
        },
      })
      return {
        ...htmlProps,
        documentary,
      }
    },
  })
  return [rule]
}

module.exports=createRules

/**
 * @typedef {import('../lib/Documentary').default} Documentary
 */

module.exports.getComponents = getComponents