const { EOL } = require('os');
const { spawn } = require('../../../stdlib');
const { indent } = require('../');

const treeRule = {
  re: /( *)%TREE (.+)%/mg,
  async replacement(match, ws, m) {
    const args = m.split(' ').map((a) => a.trim())
    const p = spawn('tree', process.platform == 'win32' ? args : ['--noreport', ...args])
    try {
      const { code, stdout } = await p.promise
      if (code ||
        (process.platform == 'win32' ? /Invalid path/.test(stdout) : /\[error opening dir\]/.test(stdout))
      ) {
        this.log('Could not generate a tree for %s', args.join(' '))
        return match
      }
      let s = codeSurround(stdout)
      s = indent(s, ws)
      return s
    } catch (err) {
      if (err.code == 'ENOENT') {
        console.warn('tree is not installed')
        return match
      }
      this.log(err.message)
      return match
    }
  },
}

const codeSurround = (m, lang = 'm') => `\`\`\`${lang}${EOL}${m.trim()}${EOL}\`\`\``

module.exports=treeRule
